/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

var variableTreeViewLog = getLoggingMgr().getLogger("variableTreeView");
//variableTreeViewLog.setLevel(LOG_DEBUG);

// Globals
function variableTreeView(dbgTabManager, context) {
    treeView.apply(this, []);
    this.mSingleSelect = true;
    this.mVariableList = [];
    this.mFrameVarList = [];
    this.context = context;
    this.debug=0;
    this.dbgTabManager = dbgTabManager;
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
variableTreeView.prototype = new treeView();
variableTreeView.prototype.constructor = variableTreeView;

variableTreeView.prototype.setContext = function(context) {
    this.context = context;
}

/* void selectionChanged(); */
variableTreeView.prototype.selectionChanged = function() {
    window.setTimeout("window.updateCommands('debugvartab_selection_changed');",1);
}

variableTreeView.prototype.haveVariable = function(expression) {
    variableTreeViewLog.debug('haveVariable '+expression);
    try {
        return this.dbgTabManager.haveWatchedVar(expression);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return false;
}

variableTreeView.prototype.addVariable = function(expression, value) {
    variableTreeViewLog.debug('addVariable '+expression+' = '+value);
    // unable to set the value, so we add a new value
    try {
        this.dbgTabManager.addWatchedVar(expression, value);
        return true;
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return false;
}

variableTreeView.prototype.setVariable = function(index, expression, value, dataType) {
    variableTreeViewLog.debug('setVariable '+expression+' = '+value);
    if (!this.dbgTabManager.session) return false;
    var state = this.dbgTabManager.session.getState();
    if (state != DBG.STATE_BREAK && state != DBG.STATE_INTERACTIVE) return false;

    var ret = false;
    try {
        //this.dbgTabManager.valueChanged(this.context, expression, value);
        var variable = this.mVariableList[index];
        if (!variable) return ret;
        if (variable.fullname != expression) {
            // they've changed the name on us, add a new variable
            this.addVariable(expression,value);
        }

        var exceptionText = null;
        try {
            if (typeof(dataType) == 'undefined') {
                dataType = null;
            }
            variable.debugprop.setValue(value, dataType);//, got_error);
        } catch (e) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                                getService(Components.interfaces.koILastErrorService);
            exceptionText = lastErrorSvc.getLastErrorMessage();
            if (!exceptionText)
                exceptionText = "unexpected error setting value";
        }
        // force the outliner to get new values
        delete variable.value;
        delete variable.type;
        this.mTree.invalidateRow(index);
        if (exceptionText) {
            dialog_alert("Error setting value: " + exceptionText);
        } else {
            ret = true;
        }
        this.dbgTabManager.updateWatchedVars();
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return ret;
}

variableTreeView.prototype.getSelectedVariables = function() {
    variableTreeViewLog.debug('getSelectedVariables');
    var selected = [];
    try {
        for (var index=0; index < this.mTotalRows; index++) {
            if (this.mSelection.isSelected(index)) {
                selected[selected.length] = this.mVariableList[index];
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return selected;
}

variableTreeView.prototype.removeSelectedVariables = function() {
    variableTreeViewLog.debug('removeSelectedVariables');
    try {
        for (var index=0; index < this.mTotalRows; index++) {
            if (this.mSelection.isSelected(index)) {
                var variable = this.mVariableList[index];
                if (variable.level == 0) {
                    if (!this.dbgTabManager.removeWatchedVarName(variable.fullname)) {
                        this.dbgTabManager.removeWatchedVarName(variable.name)
                    }
                }
            }
        }
        this.dbgTabManager.updateWatchedVars();
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
}

variableTreeView.prototype.removeVariable = function(index) {
    variableTreeViewLog.debug('removeSelectedVariables');
    try {
        // we can only remove watched variables, so assume that is what we want to do
        if (this.mVariableList[index].level == 0) {
            this.dbgTabManager.removeWatchedVarName(this.mVariableList[index].fullname);
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
}

variableTreeView.prototype.getName = function(index) {
    if (index < 0) return null;
    variableTreeViewLog.debug('getName '+index);
    try {
        if (index < this.mVariableList.length) {
            var variable = this.mVariableList[index];
            if (!variable.fullname) {
                variable.fullname = variable.debugprop.fullname;
            }
            return variable.fullname;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return null;
}

variableTreeView.prototype.getVariable = function(index) {
    if (index < 0) return null;
    variableTreeViewLog.debug('getVariable '+index);
    try {
        if (index < this.mVariableList.length) {
            var variable = this.mVariableList[index];
            if (!variable.value && variable.debugprop) {
                variable.value = variable.debugprop.value;
            }
            return variable;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return null;
}

variableTreeView.prototype.getValue = function(index) {
    if (index < 0) return null;
    variableTreeViewLog.debug('getValue '+index);
    try {
        if (index < this.mVariableList.length) {
            var variable = this.mVariableList[index];
            if (!variable.value && variable.debugprop) {
                variable.value = variable.debugprop.value;
            }
            return variable.value;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return null;
}

variableTreeView.prototype.rowCountChanged = function(start, newsize) {
    if (this.mTree) {
        this.mTree.beginUpdateBatch();
        this.mTree.rowCountChanged(start, newsize);
        this.mTree.endUpdateBatch();
        this.mTree.invalidate();
    }
}

variableTreeView.prototype.clearList = function() {
    variableTreeViewLog.debug('clearList');
    try {
        this.mVariableList = [];
        this.mFrameVarList = [];
        this.mTotalRows = 0;
        this.rowCountChanged(0,0);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
}

variableTreeView.prototype._getChildren = function (variable) {
    variableTreeViewLog.debug('_getChildren for '+variable.debugprop.fullname);
    if (!this.dbgTabManager.session) return null;
    var state = this.dbgTabManager.session.getState();
    if (state != DBG.STATE_BREAK && state != DBG.STATE_INTERACTIVE) return null;

    try {
        var children = new Object();
        // force a request for the next page of children, then grab all
        // available children. If no children have been retrieved, the fist page
        // is retrieved.
        variable.debugprop.getChildrenNextPage(children, new Object());
        variable.debugprop.getAvailableChildren(children, new Object());
        if (variable.debugprop.numchildren > children.value.length) {
            //dump("child length is "+variable.debugprop.numchildren+" we have "+children.value.length+"\n");
            // append a special var that will be used to retreive additional
            // pages of children
            children.value[children.value.length] = "...Next Page...";
        }
        return children.value;
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return null;
}

variableTreeView.prototype.dumpVariable = function(v,i) {
    variableTreeViewLog.debug('dumpVariable');
    try {
        if (!v.debugprop)
            dump("var: i: "+i+" pindex: "+v.parent+" level: "+v.level+" havechild: "+v.isContainer+"\n");
        else
            dump("var: i: "+i+" pindex: "+v.parent+" level: "+v.level+" havechild: "+v.isContainer+" name: "+v.debugprop.fullname+"\n");
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
}

variableTreeView.prototype.makeVariable = function(debugprop, parent, level) {
    //variableTreeViewLog.debug('makeVariable');
    var variable = new Object();
    try {
        var children = null;
        var placeholder = 0;
        variable.isEmpty = false;
        variable.parent = parent;
        variable.level = level;
        variable.isContainerOpen = false;
        variable.children = null;
        variable.fullname = null;
        variable.name = null;
        if (typeof(debugprop) == 'string') {
            variable.fullname = debugprop;
            variable.name = debugprop;
            variable.value = "";
            variable.type = "";
        } else {
            variable.debugprop = debugprop;
            variable.isContainer = variable.debugprop.children;
            if (variable.isContainer) {
                variable.fullname = variable.debugprop.fullname;
                variable.isContainerOpen = this.dbgTabManager.getVarState(variable.fullname);
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return variable;
}

variableTreeView.prototype.getVariableChildren = function(variable, varIndex) {
    var children = this._getChildren(variable);
    if (!children) return;
    variable.children = this.getVariables(children, varIndex, variable.level+1);
}

variableTreeView.prototype.getVariables = function(debugpropArray, parent, level) {
    var newArray = [];
    variableTreeViewLog.debug('getVariables');
    try {
        var children, variable;
        for (var i =0; i < debugpropArray.length; i++) {
            // dump('var type '+typeof(debugpropArray[i])+'\n');
            variable = this.makeVariable(debugpropArray[i], parent, level);
            newArray[newArray.length] = variable;
            if (variable.isContainerOpen) {
                this.getVariableChildren(variable, newArray.length - 1);
                if (variable.children)
                    newArray = newArray.concat(variable.children);
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return newArray;
}

variableTreeView.prototype.buildList = function(debugpropArray, frameIndex) {
    window.setCursor("wait");
    variableTreeViewLog.debug('buildList');
    try {
        this.mVariableList = this.getVariables(debugpropArray, -1, 0);
        this.mTotalRows = this.mVariableList.length;

        //dump("variableTreeView.buildList length "+this.mVariableList.length+", got "+debugpropArray.length+"\n");
        this.rowCountChanged(0,this.mVariableList.length);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    window.setCursor("auto");
}

variableTreeView.prototype.getCellText = function(row, colID) {
    var res = "";
    //variableTreeViewLog.debug('getCellText');
    try {
        if (row >= this.mVariableList.length)
            return '';

        var variable = this.mVariableList[row];
        if (!variable) return '';

        if (colID == "name") {
            if (!variable.name) {
                variable.name = variable.debugprop.name;
            }
            // yes, we still want to retreive fullname here
            if (!variable.fullname) {
                variable.fullname = variable.debugprop.fullname;
            }
            res = variable.name;
        } else if (colID == "value") {
            if (typeof(variable.value) == "undefined") {
                variable.value = variable.debugprop.value;
            }
            res = variable.value;
        } else if (colID == "type") {
            if (typeof(variable.type) == "undefined") {
                variable.type = variable.debugprop.type;
            }
            res = variable.type;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return res;
}

variableTreeView.prototype.isEditable = function(row, colID) {
    if (colID == "value") return true;
    return false;
}

variableTreeView.prototype.isContainer = function(index) {
    var res = 0;
    //variableTreeViewLog.debug('isContainer');
    try {
        if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
            res = this.mVariableList[index].isContainer;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return res;
}

variableTreeView.prototype.isContainerOpen = function(index) {
    var res = false;
    //variableTreeViewLog.debug('isContainerOpen');
    try {
        if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
            var variable = this.mVariableList[index];
            res = variable.isContainerOpen;
            if (res && ! variable.children) {
                this._addChildren(index, variable);
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return res;
}

  /* long getLevel(in long index); */
variableTreeView.prototype.getLevel = function(index) {
    //variableTreeViewLog.debug('getLevel');
    try {
        if (index < this.mVariableList.length && typeof(this.mVariableList[index]) != 'undefined') {
            return this.mVariableList[index].level;
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return 0;
}

  /* boolean hasNextSibling(in long rowIndex, in long afterIndex); */
variableTreeView.prototype.hasNextSibling = function(rowIndex, afterIndex) {
    //variableTreeViewLog.debug('hasNextSibling');
    try {
        if (afterIndex < (this.mTotalRows - 1) && typeof(this.mVariableList[rowIndex]) != 'undefined') {
            var variable = this.mVariableList[rowIndex];
            for (var i = rowIndex + 1; i < this.mTotalRows && variable.level <= this.mVariableList[i].level; i++) {
                if (variable.level == this.mVariableList[i].level) return true;
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return false;
}

/* long getParentIndex(in long rowIndex); */
variableTreeView.prototype.getParentIndex = function(index) {
    var parent = -1;
    //variableTreeViewLog.debug('getParentIndex');
    try {
        if (index < this.mVariableList.length && index > 0 && this.mVariableList[index].level > 0) {
            // parent is not going to be correct most of the time, go up the list and compare levels
            var level = this.mVariableList[index].level;
            for (var i = index - 1; i >= 0; i--) {
                if (level > this.mVariableList[i].level) {
                    parent = i;
                    break;
                }
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    return parent;
}

variableTreeView.prototype._addChildren = function(index, variable) {
    window.setCursor("wait");
    variableTreeViewLog.debug('_addChildren');
    try {
        this.getVariableChildren(variable, index);
        if (!variable.children) return;

        var fa = this.mVariableList.slice(0,index+1);
        var fe = this.mVariableList.slice(index+1);
        this.mVariableList = fa;
        this.mVariableList = this.mVariableList.concat(variable.children);
        this.mVariableList = this.mVariableList.concat(fe);
        this.mTotalRows = this.mVariableList.length;

        // keep ourselves displaying correctly after adding children.
        var firstVisibleRow = this.mTree.getFirstVisibleRow();
        var currentIndex = this.mSelection.currentIndex;
        this.rowCountChanged(0,this.mVariableList.length);
        this.mSelection.currentIndex = currentIndex;
        this.mSelection.select(currentIndex);
        this.mTree.ensureRowIsVisible(firstVisibleRow);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    window.setCursor("auto");
}

/* this function is *slower than it should be* if dealing with *deeply* nested structues
    where a lot of them are toggled open because we're splitting the array and recreating
    it on every recursion. */
variableTreeView.prototype._toggleAddChildren = function(index) {
    var variable = this.mVariableList[index];
    variableTreeViewLog.debug('_toggleAddChildren');
    try {
        if (!variable.children) {
            // children have not been retreived, get them now
            var children = this._getChildren(variable);
            if (!children) return;
            variable.children = this.getVariables(children, index, variable.level+1);
        }

        // open the container
        var fa = this.mVariableList.slice(0,index+1);
        var fe = this.mVariableList.slice(index+1);

        this.mVariableList = fa;

        // insert the children into the array
        this.mVariableList = this.mVariableList.concat(variable.children);
        if (fe.length > 0) this.mVariableList = this.mVariableList.concat(fe);

        // now we have a new index, check if any of the new children are open
        for (var i = index + 1; i < this.mVariableList.length && variable.level < this.mVariableList[i].level; i++) {
            if (this.mVariableList[i].level == variable.level + 1 &&
                this.mVariableList[i].isContainerOpen &&
                (this.mVariableList.length > i+1 &&
                 this.mVariableList[i+1].level == this.mVariableList[i].level)) {
                this._toggleAddChildren(i);
            }
        }
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
}

variableTreeView.prototype.addChildPage = function(index) {
    if (index >= this.mVariableList.length) return;
    var variable = this.mVariableList[index];
    if (!variable.isContainer) return;
    window.setCursor("wait");
    try {
        var firstVisibleRow = this.mTree.getFirstVisibleRow();
        this.mTree.beginUpdateBatch();
        if (variable.isContainerOpen) {
            // remove container children
            var level = variable.level;
            var i;
            for (i = index + 1; i < this.mVariableList.length && level < this.mVariableList[i].level; i++) { }
            this.mVariableList.splice(index+1,i-(index+1));
            variable.children = null;
            // add the children back, this forces a fetch of the next page as
            // well
            this._toggleAddChildren(index);
        }
        this.mTotalRows = this.mVariableList.length;
        this.mTree.rowCountChanged(0, this.mVariableList.length);
        this.mTree.invalidate();
        this.mTree.endUpdateBatch();
        this.mSelection.currentIndex = index;
        this.mSelection.select(index);
        this.mTree.ensureRowIsVisible(firstVisibleRow);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    window.setCursor("auto");
}

  /* void toggleOpenState(in long index); */
variableTreeView.prototype.toggleOpenState = function(index) {
    if (index >= this.mVariableList.length) return;
    var variable = this.mVariableList[index];
    if (!variable.isContainer) return;
    window.setCursor("wait");
    variableTreeViewLog.debug('toggleOpenState');
    try {
        var firstVisibleRow = this.mTree.getFirstVisibleRow();
        this.mTree.beginUpdateBatch();
        if (variable.isContainerOpen) {
            // close the container
            this.dbgTabManager.contractVar(variable.fullname);
            variable.isContainerOpen = false;
            var level = variable.level;
            var i;
            for (i = index + 1; i < this.mVariableList.length && level < this.mVariableList[i].level; i++) { }
            this.mVariableList.splice(index+1,i-(index+1));
        } else {
            this.dbgTabManager.expandVar(variable.fullname);
            variable.isContainerOpen = true;
            this._toggleAddChildren(index);
        }
        this.mTotalRows = this.mVariableList.length;
        this.mTree.rowCountChanged(0, this.mVariableList.length);
        this.mTree.invalidate();
        this.mTree.endUpdateBatch();
        this.mSelection.currentIndex = index;
        this.mSelection.select(index);
        this.mTree.ensureRowIsVisible(firstVisibleRow);
    } catch (ex) {
        variableTreeViewLog.exception(ex);
    }
    window.setCursor("auto");
}

variableTreeView.prototype.Sort = function(colId) {
    // do nothing for now
}

variableTreeView.prototype.selectIndex = function(index) {
    this.mSelection.currentIndex = index;
    this.mSelection.select(index);
    this.mTree.ensureRowIsVisible(index);
}
