/* Copyright (c) 2000-2004 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch new windows such as
 * the help system, Rx, VPM, etc.
 *
 */

var _gLaunchLog = getLoggingMgr().getLogger("run");
var _gLaunchBaseHelpURL = null;

function _launch_getBaseHelpURL()
{
    try {
        var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].getService();
        var koDirSvc = Components.classes["@activestate.com/koDirs;1"].getService();

        var localPath = osPathSvc.joinlist(6,
                                           [koDirSvc.mozBinDir, "chrome",
                                            "jaguar", "content", "doc",
                                            "komodo"]);
        var baseURL = windowManager_getMainWindow().uriparse_localPathToURI(localPath);
        return baseURL;
    } catch(ex) {
        _gLaunchLog.exception(ex, "error determining base help URL");
        return null;
    }
}

function launch_MainHelp(page) {
    if (typeof(page) == "undefined" || page == null)
        page = "index.html";

    if (!_gLaunchBaseHelpURL) {
        _gLaunchBaseHelpURL = _launch_getBaseHelpURL();
    }

    try {
        var url = _gLaunchBaseHelpURL + "/" + page;
        windowManager_getMainWindow().browse_OpenUrlInDefaultBrowser(url);
    } catch(e) {
        log.exception(e);
    }
}


var help_tag2page = {
    // Prefs panels.
    "Appearance Prefs panel": "komodo-doc-prefs.html#UI_Configuration",
    "Code Intelligence Prefs panel": "komodo-doc-prefs.html#code_intell",
    "Debugger Prefs panel": "komodo-doc-prefs.html#Debugger",
    "Editor Prefs panel": "komodo-doc-prefs.html#prefs_editor",
    "Key Bindings Prefs panel": "komodo-doc-prefs.html#Config_Key_Bindings",
    "Indentation Prefs panel": "komodo-doc-prefs.html#Indentation_Features",
    "Smart Editing Prefs panel": "komodo-doc-prefs.html#smart_editing",
    "Save Options Prefs panel": "komodo-doc-prefs.html#Save_Options",
    "File Associations Prefs panel": "komodo-doc-prefs.html#File_Associations",
    "Fonts and Colors Prefs panel": "komodo-doc-prefs.html#Customizing_Fonts",
    "GUI Builder Prefs panel": "komodo-doc-prefs.html#gui_builder",
    "Interactive Shell Prefs panel": "komodo-doc-prefs.html#int_shell",
    "Internationalization Prefs panel": "komodo-doc-prefs.html#Internationalization",
    "Language Help Prefs panel": "komodo-doc-prefs.html#language_help",
    "Languages Prefs panel": "komodo-doc-prefs.html#lang_config",
    "Perl Prefs panel": "komodo-doc-prefs.html#Perl",
    "PHP Prefs panel": "komodo-doc-prefs.html#PHP",
    "Python Prefs panel": "komodo-doc-prefs.html#Python",
    "Tcl Prefs panel": "komodo-doc-prefs.html#Tcl",
    //XXX This is prefs panel is Tcl-specific although its title doesn't
    //    suggest so. If the other languages grow in a similar way then
    //    will have to extend the Prefs panel name -> Help tag system.
    "Syntax Checking Prefs panel": "komodo-doc-prefs.html#Tcl",
    "HTML Prefs panel": "komodo-doc-prefs.html#html",
    "New Files Prefs panel": "komodo-doc-prefs.html#new_files",
    "Printing Prefs panel": "komodo-doc-prefs.html#printing",
    "Projects and Workspace Prefs panel": "komodo-doc-prefs.html#projects",
    "Proxies Prefs panel": "komodo-doc-prefs.html#Proxy",
    "Servers Prefs panel": "komodo-doc-prefs.html#Servers",
    "Shared Support Prefs panel": "komodo-doc-prefs.html#Shared_Support",
    "Source Code Control Prefs panel": "komodo-doc-prefs.html#scc",
    "CVS Prefs panel": "komodo-doc-prefs.html#CVS",
    "Perforce Prefs panel": "komodo-doc-prefs.html#Perforce",
    "Web & Browser Prefs panel": "komodo-doc-prefs.html#web",
    "Windows Integration Prefs panel": "komodo-doc-prefs.html#integration",

    "Save Remotely As": "komodo-doc-files.html#Saving_Files_Remotely",
    "Open Remote File": "komodo-doc-files.html#Opening_Remote_Files",
    "Current File Settings": "komodo-doc-files.html#files_settings",
    "Find/Replace": "komodo-doc-search.html#Finding_a_word",
    "Find in Files": "komodo-doc-search.html#find_in_files",
    "Import from File System": "komodo-doc-project.html#project_impcomp",
    "Snippet Properties": "komodo-doc-snippets.html#snippets_options",
    "Command Properties": "komodo-doc-run.html#runcmd_props",
    "URL Properties": "komodo-doc-urls.html#urls_options",
    "Directory Shortcut Properties": "komodo-doc-openshort.html#openshort_options",
    "Template Properties": "komodo-doc-templates.html#templates_options",
    "Macro Properties": "komodo-doc-macros.html#macros_options",
    "Run Command": "komodo-doc-run.html#runcmd_props",
    "Object Browser":"komodo-doc-codeintel.html#codeintel_objectbrowser",
    "Debugging Options": "komodo-doc-debugger.html#Debugging_Options",
    "New File": "komodo-doc-files.html#Creating_a_New_File",
    "Breakpoint/Spawnpoint Properties": "komodo-doc-debugger.html#breakpoints_and_spawnpoints",
    "Custom Menu/Toolbar Properties": "komodo-doc-custtbi.html#custtbi_props"

    // How about these:
    // (none)
    // Rejected for now (URLs not necessarily correct):
    // - "Breakpoints Tab": "komodo-doc-debugger.html#breakpoints_and_spawnpoints",
    // - "Projects Tab": "komodo-doc-project.html#Managing_Projects",
    // - "Toolbox Tab": "komodo-doc-toolbox.html",
    // - "Interactive Shell": "komodo-doc-intshell.html",
};

function launch_getHelpTag(tag) {
    if (tag in help_tag2page && help_tag2page[tag] != null) {
        return help_tag2page[tag];
    }
    return null;
}
// Open Komodo's help to the appropriate page for the given "tag" name.
// A tag is some name that identifies a thing in Komodo on which there is
// specific help. The intention is to decouple the actual Komodo help
// URL and anchor from the association "thing". The mapping is hardcoded
// here.
function launch_MainHelpTag(tag) {
    var helpTag = launch_getHelpTag(tag)
    if (helpTag) {
        launch_MainHelp(helpTag);
    } else {
        var msg = "Error opening context-sensitive help page.";
        dialog_internalError(msg, msg+"\nUnrecognized tag: '"+tag+"'.");
        launch_MainHelp();
    }
}


function launch_Tutorials(tutorial /* =null */)
{
    // Open the Tutorial in the help browser
    //    "tutorial" can optionally be used to specify a particular tutorial
    //        to start with. Currently valid values are "perl", "python",
    //        "php", "xslt", and null (to go to Tutorial home page).
    //XXX Shane changed that "home page" to "showcase" from null. Change that
    //    back. --TM
    var page;
    if (tutorial == 'showcase') {
        page = "tutorial/tourlet_over.html";
    } else {
        page = "tutorial/"+tutorial+"tut.html";
    }
    launch_MainHelp(page);
}


function launch_TipOfTheDay() {
    openWindowUniqueInstance("chrome://jaguar/content/tipoftheday/tipoftheday.xul",
                             "komodo_tip",
                             "chrome,dialog,dependent,resizable");
}


function launch_LanguageHelp() {
    // Get the current document's language.
    var language = null;
    var view = gViewMgr.currentView;
    if (view != null) {
        language = view.document.language;
    }

    // Get the help command appropriate for that language.
    var command=null, name=null;
    if (language && gPrefs.hasStringPref(language+'HelpCommand')) {
        command = gPrefs.getStringPref(language+'HelpCommand');
        if (command) {
            name = language + " Help";
        }
    }
    if (!command) {
        // Couldn't find language-specific help command -- use the default one.
        command = gPrefs.getStringPref('DefaultHelpCommand');
        name = "Help";
    }

    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   name); // name
}


function launch_AlternateLanguageHelp() {
    var command = gPrefs.getStringPref('OtherHelpCommand');
    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   "Alternate Help"); // name
}


function launch_Find(searchTerm) {
    _launch_FindTab("find", searchTerm);
}

function launch_Replace() {
    _launch_FindTab("replace", null);
}

var gFindDialogPanel = null;
var gFindSearchTerm = null;
function _launch_FindTab(panel, searchTerm) {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while find.xul is loading. The find dialog will retrieve these
    // contents when it is ready.
    InputBuffer_Start();
    gFindSearchTerm = searchTerm;
    gFindDialogPanel = panel; // special global to pass info to find.xul
    window.openDialog("chrome://jaguar/content/find/find.xul",
                      "komodo_find",
                      "chrome,dialog,dependent,close=yes");
}

function launch_FindInFiles(searchTerm, folders, fileTypes) {
    _launch_FindInFilesTab("find", searchTerm, folders, fileTypes);
}







var gFindInFilesDialogPanel = null;
var gFindInFilesCwd = null;
var gFindInFilesSearchTerm = null;
var gFindInFilesFolders = null;
var gFindInFilesFiletypes = null;
function _launch_FindInFilesTab(panel, searchTerm, folders, fileTypes) {
    // Transfer focus to the hidden input buffer to capture keystrokes from
    // the user while findInFiles.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    InputBuffer_Start();

    gFindInFilesDialogPanel = panel; // special global to pass info to findInFiles.xul
    gFindInFilesSearchTerm = searchTerm;
    gFindInFilesFolders = folders;
    gFindInFilesFiletypes = fileTypes;
    var view = gViewMgr.currentView;
    if (view != null &&
        view.getAttribute("type") == "editor" &&
        view.document.file.isLocal) {
        gFindInFilesCwd = view.document.file.dirName;
    } else {
        gFindInFilesCwd = null;
    }
    window.openDialog("chrome://jaguar/content/find/findInFiles.xul",
                      "komodo_find_in_files",
                      "chrome,dialog,dependent,close=yes");
}


function launch_RunCommand() {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while run.xul is loading.  (Get the current view before calling
    // inputbuffer start so we have the correct focus coming out of the
    // dialog.)
    InputBuffer_Start();
    openWindowUniqueInstance("chrome://jaguar/content/run/run.xul",
                             "komodo_run",
                             "chrome,dialog,dependent");
}


function launch_DiffWindow(diff, title /* ="Diff Output" */)
{
    if (typeof(title) == "undefined") {
        title = "Diff Output";
    }
    var obj = new Object();
    obj.title = title;
    obj.text = diff;
    obj.warning = null;
    if (typeof(diff) != "string") {
        obj.text = diff.result;
        if (diff.doc1.existing_line_endings !=
            diff.doc2.existing_line_endings) {
            obj.warning = "The line endings in the files are different, but "+
                    "this diff does not reflect differences in line endings.";
        }
        // This is a temporary quick fix HACK for bug 36269.
        obj.cannotHandleF9 = true;
    } else {
        obj.cannotHandleF9 = false;
    }
    obj.language = 'Diff';
    obj.dimensionPrefPrefix = "sccDiffWin-";
    obj.processDiffs = true;
    openWindowUniqueInstance("chrome://jaguar/content/simpleeditor/simpleeditor.xul",
                             "diff-window",
                             "chrome,dialog=no,resizable=yes,scrollbars,close=yes",
                             obj);
}


function launch_watchLocalFile() {
    var filename = filepicker_openFile();
    if (filename)
        openWindowMultipleInstance("chrome://jaguar/content/tail/tail.xul",
                                            "komodo_tail",
                                            "chrome,resizable,scrollbars,dialog=no",
                                            filename,
                                            window);
}


function launch_rx() {
    var obj = new Object();
    //XXX We _could_ send in some configuration data on 'obj', but aren't
    //    yet. See the top of rxx.js for the API.
    openWindowUniqueInstance("chrome://jaguar/content/rxx/rxx.xul",
                             "komodo_rxx",
                             "chrome,resizable,dialog=no,close=yes,dependent=no",
                             obj);
}


gObjectBrowserSymbol = null;
gObjectBrowserModule = null;
gObjectBrowserExact = null;
gObjectBrowserLanguage = null;
gObjectBrowserShowImports = null;
function launch_objectBrowser(symbol, module, exact, language) {
    try {
        var obj = new Object();
        if (typeof(symbol) == 'undefined') {
            symbol = '';
        }
        if (typeof(module) == 'undefined') {
            module = '';
        }
        if (typeof(exact) == 'undefined') {
            exact = false;
        }
        if (typeof(language) == 'undefined') {
            language = '';
        }
        gObjectBrowserSymbol = symbol;
        gObjectBrowserModule = module;
        gObjectBrowserExact = exact;
        gObjectBrowserLanguage = language;
        gObjectBrowserShowImports = null;

        var _WMSvc = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                            .getService(Components.interfaces.nsIWindowMediator);
        var existingWindow = _WMSvc.getMostRecentWindow("komodo_objectbrowser");
        if (existingWindow) {
            existingWindow.Reload();
        } else {
            openWindowUniqueInstance("chrome://jaguar/content/codeintel/objectbrowser.xul",
                                     "komodo_objectbrowser",
                                     "chrome,resizable,dialog=no,close=yes,dependent=no");
        }
    } catch (e) {
        log.exception(e);
    }
}


function launch_perlappWindow() {
    var sysDataSvc = Components.classes["@activestate.com/koSystemDataService;1"]
                     .getService(Components.interfaces.koISystemDataService);

    var versionStr = parseFloat(sysDataSvc.getString("pdk.version"));
    openWindowUniqueInstance("chrome://jaguar/content/pdk/perlapp.xul",
                "komodo_perlapp",
                "chrome,resizable,dialog=no",
                versionStr);
}



//---- VPM stuff
function launch_VPMWindow() {
    var vpmSvc = Components.classes["@activestate.com/koVPMService;1"].
                 getService(Components.interfaces.koIVPMService);

    // Basically launches: vpm --showurl --noquit
    var msg = null;
    try {
        msg = vpmSvc.start();
    } catch (e) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                            getService(Components.interfaces.koILastErrorService);
        dialog_alert("The Perl Visual Package Manager (VPM) failed to launch.",
                     lastErrorSvc.getLastErrorMessage());
        return;
    }
    if (msg) {
        dialog_alert(msg);
    }
}



//---- Input buffering
// When you need to capture user input while a slow XUL window is loading you
// can use the input buffer. Usage:
//  - in some JS code:
//      InputBuffer_Start()
//      // open XUL window
//
//  - in slow XUL window onload handler:
//      var contents = InputBuffer_Finish();
//      // use the contents somehow
//

var _gInputBuffer_active;

function InputBuffer_Start()
{
    _gInputBuffer_active = 1;
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    inputBufferWidget.focus();
}

function InputBuffer_OnFocus(event)
{
    // if it is not active the hidden input buffer should not have the focus
    if (!_gInputBuffer_active && gViewMgr.currentView) {
        // This has to be in a timeout for the controllers to work right.
        window.setTimeout('gViewMgr.currentView.setFocus();', 1)
    }
}


function InputBuffer_Finish()
{
    // Return the contents of the input buffer and stop buffering.
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    var contents = inputBufferWidget.value;
    inputBufferWidget.value = "";

    _gInputBuffer_active = 0;
    return contents;
}


