
function MacroManager() {
    this.mode = 'stopped';
    this._currentMacro = new Array();
    this._currentMacroText = '';
    this.log = getLoggingMgr().getLogger('MacroManager');
}


MacroManager.prototype = new Object();
MacroManager.prototype.constructor = MacroManager;

MacroManager.prototype.startRecording = function() {
    if (this.mode != 'paused') {
        if (this._currentMacro) {
            delete this._currentMacro;
        }
        this._currentMacro = new Array();
        this._currentMacro.push("// Macro recorded on " + Date() + '\n');
        var macroVersionNumber = "2"; // must match the number in peMacro.js
        this._currentMacro.push('komodo.assertMacroVersion(' + macroVersionNumber + ');\n');
        this._currentMacro.push("if (komodo.view) { komodo.view.setFocus() };\n");
        this._currentMacroText = new Array();
    }
    this.mode = 'recording';
    var record = document.getElementById("macroRecord");
    window.updateCommands("macro");
    StatusBar_AddMessage("Recording Macro", "macro", 0, true)
}

MacroManager.prototype.stopRecording = function(quiet /* false */) {
    this.mode = 'stopped';
    this._finishTextMacro();
    if (typeof(quiet) == 'undefined') {
        quiet = false;
    }
    window.updateCommands("macro");
    if (!quiet) {
        StatusBar_AddMessage(null, "macro", 0, false)
        StatusBar_AddMessage("Macro Recorded", "macro", 3000, true)
    }
}

MacroManager.prototype.pauseRecording = function() {
    this.stopRecording();  // but...
    this.mode = 'paused'; // set the mode to paused
}

MacroManager.prototype.suspendRecording = function() {
    // This is used to temporarily suspend recording when
    // recording a macro that executes another macro -- we don't
    // want what that macro does (i.e. launching of commands) to
    // be recorded in _this_ macro.  This must be paired with
    // a call to resumeRecording (below)
    this.stopRecording(true); // but ...
    this.mode = 'suspended'; // set the mode to paused
}

MacroManager.prototype.resumeRecording = function() {
    if (this.mode == 'suspended') {
        this.mode = 'paused';
        this.startRecording();
    }
}


MacroManager.prototype._finishTextMacro = function() {
    if (this._currentMacroText != '') {
        this._currentMacro.push("komodo.view.selection = '" +
                                this._currentMacroText.replace('\\', '\\\\', 'g').replace("\'", "\\'", 'g') +
                                "';\n");
        this._currentMacroText = '';
    }
}

MacroManager.prototype.appendCommand = function(command) {
    try {
        if (! this._currentMacro) {
            this.log.warn("Trying to append a command while not in a macro!!!");
            return;
        }
        this._finishTextMacro();
        this._currentMacro.push("komodo.doCommand('" + command +"')\n");
    } catch (e) {
        log.exception(e);
    }
}

MacroManager.prototype.recordKeyPress = function(c) {
    if (! this._currentMacro) {
        this.log.warn("Trying to record keypresses while not in a macro!!!");
        return;
    }
    this._currentMacroText += c;
}

MacroManager.prototype.undo = function() {
    if (this._currentMacroText != '') {
        this._currentMacroText = this._currentMacroText.slice(0, this._currentMacroText.length-1);
    } else {
        this._currentMacro.pop();
    }
}

MacroManager.prototype.appendCode = function(code) {
    if (! this._currentMacro) {
        this.log.warn("Trying to append code while not in a macro!!!");
        return;
    }
    this._finishTextMacro();
    this._currentMacro.push(code);
}

MacroManager.prototype.executeMacro = function(macro) {
    var view = gViewMgr.currentView;
    var scin = null;

    if (view && view.getAttribute('type') == 'editor') {
        scin = view.scimoz;
        scin.beginUndoAction();
    }

    try {
        try {
            if (macro == null) {
                macro = this._currentMacro;
            }
            if (macro == null) return;
            if (this.mode == 'recording') {
                this.mode = 'stopped'; // avoid recursion!
            }
            if (scin) {
                // XXX weird stuff.
                // For some reason, Scintilla always thinks the lines are folded, which results in selection of
                // folded lines, which is typically not what we want So we unfold all the lines in the buffer.
                for (var lineno=0;lineno < scin.lineCount;lineno++)
                    if ((scin.getFoldLevel(lineno) & scin.SC_FOLDLEVELHEADERFLAG) &&
                        !scin.getFoldExpanded(lineno))
                            scin.toggleFold(lineno)
            }
            var code = (macro.join(''));
            macro_evalAsJavascript(code);
        } catch (e) {
            this.log.error(e);
        }
    } finally {
        if (scin) {
            scin.endUndoAction();
        }
    }
}

MacroManager.prototype.executeLastMacro = function(macro) {
    StatusBar_AddMessage("Executing Last Recorded Macro", "macro", 3000, false)
    this.executeMacro(this._currentMacro);
}

MacroManager.prototype.saveToToolbox = function(macro) {
    var name = dialog_prompt("Macro Name",
                             "Enter name for new macro:");
    if (!name) return;
    var part = Components.classes['@activestate.com/koPart?type=macro;1']
               .createInstance(Components.interfaces.koIPart_macro);
    part.setStringAttribute('name', name);
    part.setStringAttribute('language', 'JavaScript');
    part.setBooleanAttribute('async', false);
    part.setBooleanAttribute('trigger_enabled', false);
    part.setStringAttribute('trigger', "trigger_postopen");  // just to have something
    part.value = stringutils_escapeWhitespace(this._currentMacro.join(''));
    toolboxMgr.addItem(part);
    uilayout_ensureTabShown('toolbox_tab');
}

var gMacroMgr = new MacroManager();
