/* Copyright (c) 2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Functions for use less common dialogs (common dialog helpers go in
 * dialogs.js).
 *
 * Summary of Dialogs:
 *  dialog_pickPreview      Select a URL to preview the given file.
 *  dialog_invalidLicense   Display relevant invalid license text and options.
 *
 */

//---- public methods

// Pick a preview File/URL for the given file.
//
//  "url" indicate the file to preview.
//  "language" is the language of the current file. If this is not specified it
//      will be automaticxally determined from the URL.
//  "mode" is one of "previewing" or "setting". The UI is slightly different
//      depending on whether the preview file/URL is being sought to
//      immediately _use_ it or to just set it. The default is "previewing".
//
// If the dialog is cancelled it returns null. Otherwise it returns an object
// with the following attributes:
//      .preview    The file or URL to use to preview the given URL. Note
//                  that this may be the given URL itself.
//      .remember   A boolean indicating if the current setting should be
//                  remembered so this picker need not be called again.
//
function dialog_pickPreview(url, language /*=null*/, mode /*="previewing"*/)
{
    if (typeof(url) == 'undefined' || url == null)
        throw("Must specify 'url' argument to dialog_pickPreview().");
    if (typeof(language) == 'undefined') language = null;
    if (typeof(mode) == 'undefined' || mode == null) mode = "previewing";

    // Show the dialog.
    var obj = new Object();
    obj.url = url;
    obj.language = language;
    obj.mode = mode;
    window.openDialog("chrome://jaguar/content/dialogs/pickPreview.xul",
                      "_blank",
                      "chrome,modal,titlebar",
                      obj);
    if (obj.retval == "Cancel") {
        return null
    } else {
        return obj;
    }
}


// Display the invalid license dialog and offer options.
//
// Arguments:
//  "message" is message to show.
//  "buttons" is a list of strings, each a label of a button to show.
//      Currently this is limited to three buttons, plus an optional "Close"
//      button.
//  "infos" is a list of strings identifying what information blocks to show.
//      Valid values are "beta", "purchase", "winnt", "aspn".
//  "response" is the default response. This button is shown as the default.
//      It must be one of the strings in "buttons" or empty, in which case the
//      first button is the default.
//  "title" is the dialog title.
//
// Returns the name of the button pressed, i.e. one of the strings in
// "buttons", or "Close" if the dialog was cancelled (it is possible the
// cancel a dialog without a cancel button).
//
//TODO:
// - Add accesskey support to the custom buttons: if a char is preceded by an
//   ampersand then use that for the access key. See
//   http://www.mozilla.org/projects/ui/accessibility/accesskey.html
//
function dialog_invalidLicense(message, buttons, infos, response, title)
{
    if (typeof(message) == 'undefined') message = null;
    if (typeof(buttons) == 'undefined') buttons = null;
    if (typeof(infos) == 'undefined') infos = null;
    if (typeof(response) == 'undefined') response = null;
    if (typeof(title) == 'undefined') title = null;

    // Show the dialog.
    var obj = new Object();
    obj.message = message;
    obj.buttons = buttons;
    obj.infos = infos;
    obj.response = response;
    obj.title = title;
    window.openDialog("chrome://jaguar/content/dialogs/invalidLicense.xul",
                      "_blank",
                      "chrome,modal,titlebar",
                      obj);

    return obj.response;
}


