/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* objectTimers allows you to set a window timeout or timeinterval
   on a method of an instance of an object.  This works with js
   objects or xbl objects :)

   example usage (within a class)

someobject.prototype.startInterval() {
    this._repeater = new objectTimer(this,this.doRepeater,['test string',12345]);
    this._repeater.startInterval(20);
}

someobject.prototype.stopInterval() {
    if (!this._repeater) return;
    this._repeater.stopInterval();
    this._repeater.free();
    this._repeater = null;
}

someobject.prototype.doRepeater(tag, value) {
    // do something here.
}


*/

var gObjectTimers = {};
function objectTimer(instance, func, args) {
    this.name = _objectTimer_generateTimerInternalName();
    this.timer = 200;
    this.instance = instance;
    this.func = func;
    this.args = args;
    this.running = false;
    this.timeout = null;
    this.interval = null;
    gObjectTimers[this.name] = this;
    this.cmdstring = 'gObjectTimers["'+this.name+'"].monitor();'
}

objectTimer.prototype.free = function() {
    delete gObjectTimers[this.name];
}

objectTimer.prototype.monitor  = function() {
    this.func.apply(this.instance, this.args);
}

objectTimer.prototype.startTimeout  = function(timer) {
    if (this.running) return;
    this.timer = timer;
    try {
        this.timeout = window.setTimeout(this.cmdstring, this.timer);
    } catch(e) {
        log.exception(e,'objectTimer.startTimeout exception');
        this.running = false;
        return;
    }
    this.running = true;
}

objectTimer.prototype.stopTimeout  = function() {
    if (!this.timeout) return;
    window.clearTimeout(this.timeout);
    this.timeout = null;
    this.running = false;
}

objectTimer.prototype.stop  = function(timer) {
    this.stopTimeout();
    this.stopInterval();
}

objectTimer.prototype.startInterval  = function(timer) {
    if (this.running) return;
    this.timer = timer;
    try {
        this.interval = window.setInterval(this.cmdstring, this.timer);
    } catch(e) {
        log.exception(e,'objectTimer.startInterval exception');
        this.running = false;
        return;
    }
    this.running = true;
}

objectTimer.prototype.stopInterval  = function() {
    if (!this.interval) return;
    window.clearInterval(this.interval);
    this.interval = null;
    this.running = false;
}

function _objectTimer_generateTimerInternalName() {
  var now = new Date();
  var rand = Math.floor(  1000000 * Math.random(now.getTime())  );
  return ('_objectTimer_' + rand + 's');
}
