/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// The "command" project extension.
//
// This extension manages "run command" project entries.
//

//--- Register the extension.

registeredExtensions[registeredExtensions.length] = new peCommand();


//---- The extension implementation.

function peCommand() {
    this.name = 'peCommand';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peCommand.prototype.constructor = peCommand;

peCommand.prototype.init = function() {
}

peCommand.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_runCommand', this);
}

peCommand.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_command,
                                     'ondblclick', this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_command,
                                     'onUpdatePart', this);
}

peCommand.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_command,
                                    'Run', 'cmd_runCommand',
                                    null,
                                    null,
                                    true);
}

peCommand.prototype.ondblclick = function(item,event) {
    if (item.type != 'command') return;
    this._runCommand(item)
}

peCommand.prototype.supportsCommand = function(command, part) {
    var view = gFocusedProjectView;
    if (!view) return false;
    //dump("supportsCommand for " + command +'\n');
    switch (command) {
    case 'cmd_runCommand':
        var items = gFocusedProjectView.getSelectedItems();
        if (items.length == 1 && items[0].part
            && items[0].part.type == 'command') {
            return true;
        } else {
            return false;
        }
    default:
        break;
    }
    return false;
}

peCommand.prototype.isCommandEnabled = peCommand.prototype.supportsCommand;

peCommand.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_runCommand':
        item = gFocusedProjectView.getSelectedItem();
        this._runCommand(item);
        break;
    default:
        break;
    }
}


function peCommand_addCommand(item)
{
    var part = Components.classes['@activestate.com/koPart?type=command;1']
               .createInstance(Components.interfaces.koIPart_command);
    part.type = 'command';
    part.setStringAttribute('name', "New Command");
    var obj = new Object();
    obj.part = part;
    openWindowUniqueInstance(
        "chrome://jaguar/content/run/commandproperties.xul",
        "komodo_commandproperties",
        "chrome,close=yes,modal=yes,dependent=yes", obj);
    if (obj.retval == "OK") {
        var index = -1;
        if (typeof(item)=='undefined' || !item)
            item = gFocusedProjectView.getSelectedItem();
        if (item) index = gFocusedProjectView.getIndexByPart(item);
        gFocusedProjectView.manager.addItem(part,index);
    }
}

function command_editProperties(item)
{
    var obj = new Object();
    obj.part = item.part;
    obj.item = item;
    openWindowMultipleInstance(
        "chrome://jaguar/content/run/commandproperties.xul",
        "komodo_commandproperties",
        "chrome,close=yes,dependent=yes", obj);
}

function Run_CommandPart(cmdPart) {
    // Have to guard against new (since Komodo 2.0.0 release) command
    // attributes not being defined.
    var parseOutput = null;
    if (cmdPart.hasAttribute("parseOutput")) {
        parseOutput = cmdPart.getBooleanAttribute("parseOutput");
    }
    var parseRegex = null;
    if (cmdPart.hasAttribute("parseOutput")) {
        parseRegex = cmdPart.getStringAttribute("parseRegex");
    }
    var showParsedOutputList = null;
    if (cmdPart.hasAttribute("parseOutput")) {
        showParsedOutputList = cmdPart.getBooleanAttribute("showParsedOutputList");
    }
    var clearOutputWindow = true;
    var terminationCallback = null;
    var saveInMRU = true;
    var saveInMacro = false;
    Run_RunCommand(
        window,
        cmdPart.value,
        cmdPart.getStringAttribute("cwd"),
        stringutils_unescapeWhitespace(cmdPart.getStringAttribute("env"), '\n'),
        cmdPart.getBooleanAttribute("insertOutput"),
        cmdPart.getBooleanAttribute("operateOnSelection"),
        cmdPart.getBooleanAttribute("doNotOpenOutputWindow"),
        cmdPart.getStringAttribute("runIn"),
        parseOutput,
        parseRegex,
        showParsedOutputList,
        cmdPart.getStringAttribute("name"),
        clearOutputWindow,
        terminationCallback,
        saveInMRU,
        saveInMacro
    );
    macro_recordPartInvocation(cmdPart);
}

peCommand.prototype._runCommand = function(item)
{
    Run_CommandPart(item.part);
}

peCommand.prototype.onUpdatePart = function(part,event)
{
    if (part.type == 'command') {
        part.renameable = true;
        part.iconstyle['command'] = 1;
    }
}

function AddCommandToToolbox(command, cwd, env, insertOutput,
                             operateOnSelection, doNotOpenOutputWindow, runIn,
                             parseOutput, parseRegex, showParsedOutputList,
                             name /* default=command */ )
{
    var part = Components.classes['@activestate.com/koPart?type=command;1']
                .createInstance(Components.interfaces.koIPart_command);
    part.type = 'command';
    part.value = command;
    if (typeof(name) == 'undefined' || ! name) {
        name = command;
    }
    part.setStringAttribute('name', name);
    part.setStringAttribute('cwd', cwd);
    part.setStringAttribute('env', stringutils_escapeWhitespace(env));
    part.setBooleanAttribute('insertOutput', insertOutput);
    part.setBooleanAttribute('operateOnSelection', operateOnSelection);
    part.setBooleanAttribute('doNotOpenOutputWindow', doNotOpenOutputWindow);
    part.setStringAttribute('runIn', runIn);
    part.setBooleanAttribute('parseOutput', parseOutput);
    part.setStringAttribute('parseRegex', parseRegex);
    part.setBooleanAttribute('showParsedOutputList', showParsedOutputList);
    toolboxMgr.addItem(part);
    uilayout_ensureTabShown('toolbox_tab');
}
