/* Copyright (c) 2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/*
 *
 * Komodo's Snippet Properties and "Add Snippet" dialog.
 *
 */

var log = getLoggingMgr().getLogger("snippetProperties");


var snippetname, snippetvalue;
var gApplyButton, gOKButton;
var indentCheckbox, keybinding, gPart, gItem;
var gCaretPos;
var gSetSelectionCheckbox;
var scin;
var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                    getService(Components.interfaces.koIEncodingServices);
var ANCHOR_MARKER = '!@#_anchor';
var CURRENTPOS_MARKER = '!@#_currentPos';
var gChromeWindowView = null;

function onLoad(event) {
    try {
        trace_onload();
        command_onload();
        var dialog = document.getElementById("dialog-snippetproperties");

        // "steal" the commands and keyset from the main window
        // Doing it the 'right' way takes a long time - I'm hoping this
        // will be faster.
        document.getElementById('dialog-snippetproperties').
            appendChild(opener.document.getElementById('cmdset_currentview_is_editor').
                        cloneNode(1));
        doContextMenuHackForPlugin();
        gChromeWindowView = event.view;

        dialog.appendChild(opener.document.getElementById('allcommands').
                        cloneNode(1));

        gApplyButton = dialog.getButton("extra1");
        gApplyButton.setAttribute('label', 'Apply');
        gApplyButton.setAttribute('accesskey', 'A');
        gOKButton = dialog.getButton("accept");

        gItem = window.arguments[0].item;
        gPart = gItem.part;

        if (window.arguments[0].task == 'new') {
            window.title = "Create New snippet";
            gApplyButton.setAttribute('collapsed', 'true');
        } else {
            window.title = "Snippet Properties";
        }

        snippetname = document.getElementById('snippetname');
        snippetnamelabel = document.getElementById('snippetnamelabel');
        snippetvalue = document.getElementById('snippetvalue');
        indentCheckbox = document.getElementById('indent_relative');
        gSetSelectionCheckbox = document.getElementById('set_selection');


        snippetname.value = gPart.getStringAttribute('name');

        keybinding = document.getElementById('keybindings');
        keybinding.gKeybindingMgr = opener.gKeybindingMgr;
        keybinding.part = gPart;

        keybinding.commandParam = gPart.id;
        var text = gPart.value;
        text = opener.stringutils_unescapeWhitespace(text, '\n');
        var language = 'Text';

        snippetvalue.initWithBuffer(text, language);

        scin = snippetvalue.scimoz;

        scin.caretFore = 0x0000ff;
        scin.setSelBack(1, 11184895);
        scin.caretWidth = 2;
        gSetSelectionCheckbox.setAttribute('checked',
                                           gPart.getStringAttribute('indent_relative'));
        var anchor = text.indexOf(ANCHOR_MARKER);
        var currentPos = text.indexOf(CURRENTPOS_MARKER);
        if (anchor < currentPos) {
            anchor = text.indexOf(ANCHOR_MARKER);
            text = text.replace(ANCHOR_MARKER, '');
            currentPos = text.indexOf(CURRENTPOS_MARKER);
            text = text.replace(CURRENTPOS_MARKER, '');
        } else {
            currentPos = text.indexOf(CURRENTPOS_MARKER);
            text = text.replace(CURRENTPOS_MARKER, '');
            anchor = text.indexOf(ANCHOR_MARKER);
            text = text.replace(ANCHOR_MARKER, '');
        }
        snippetvalue.setBufferText(text);
        scin.anchor = scin.positionAtChar(0,anchor);
        scin.currentPos = scin.positionAtChar(0,currentPos);
        indentCheckbox.setAttribute('checked', gPart.getStringAttribute('indent_relative'));
        gSetSelectionCheckbox.setAttribute('checked', gPart.getStringAttribute('set_selection'));
        keybinding.init();
        keybinding.updateCurrentKey();

        update_icon(gPart.iconurl);

        snippetname.focus();
        scin.focus = true; // show the caret
        scin.caretPeriod = 0;
        snippetname.select();
        //centerWindowOnScreen();

        UpdateField('name', true);

        updateOK();
    } catch (e) {
        log.exception(e);
    }
}


function OK()  {
    if (Apply()) {
        window.arguments[0].res = true;
        if (window.arguments[0].task == 'new') {
            var parentitem = window.arguments[0].parentitem;
            var item = window.arguments[0].item;
            var gFocusedProjectView = window.arguments[0].gFocusedProjectView;
            var index = gFocusedProjectView.getIndexByPart(parentitem);
            gFocusedProjectView.manager.addItem(item.part,index);
        }
        return true;
    }
    return false;
}


// Return true iff the current position and anchor position are valid.
//
// If the selection position(s) are to be maintain for snippet insertion
// they must not be within an interpolation code block because the final
// position is then ambiguous.
//
// If the positions are not valid an alert dialog is raised and the
// positions are automatically moved to appropriate valid positions.
//
function _validateSelection()
{
    var text = scin.text;
    var anchor = scin.anchor;
    var currentPos = scin.currentPos;
    var blockOffsets = Interpolate_getBlockOffsets(text, true);

    // Detemine if the positions are valid. If not, determine appropriate
    // relocations for them.
    var i, start, end;
    var valid = true;
    for (i=0; i+1 < blockOffsets.length; i+=2) {
        start = blockOffsets[i];
        end = blockOffsets[i+1];
        if (start < currentPos && currentPos < end) {
            valid = false;
            if (currentPos < anchor) { // move back
                currentPos = start;
            } else if (currentPos == anchor) { // move both back
                currentPos = start;
                anchor = start;
            } else { // move forward
                currentPos = end;
            }
        }
        if (start < anchor && anchor < end) {
            valid = false;
            if (anchor < currentPos) { // move back
                anchor = start;
            } else if (anchor == currentPos) { // move both back
                anchor = start;
                currentPos = start;
            } else { // move forward
                anchor = end;
            }
        }
    }

    // If not valid, ask the user if should readjust.
    if (!valid) {
        var query = null;
        if (anchor == currentPos) {
            query = "The cursor is in the middle of a shortcut block. " +
                    "This is not allowed. Would you like Komodo to " +
                    "adjust the cursor position to an appropriate position?";
        } else {
            query = "The selection partially overlapped with a shortcut " +
                    "block. This is not allowed. Would you like Komodo to " +
                    "adjust the selection appropriatly?";
        }
        var readjust = dialog_okCancel(query,
                                       null, // default
                                       null, // extra text
                                       null, // title
                                       "readjust_snippet_selection");
        if (readjust == "OK") {
            scin.currentPos = currentPos;
            scin.anchor = anchor;
            valid = true;
        }
    }

    return valid;
}


// Apply the current changes and return true iff successful. Otherwise return
// false.
function Apply() {
    // Validate snippet data.
    if (gSetSelectionCheckbox.getAttribute('checked') == 'true') {
        var valid = _validateSelection();
        if (!valid) {
            return false;
        }
    }

    // Apply the keybinding.
    try {
        var retval = keybinding.apply(); // This may return false if a keybinding is partially entered
        if (!retval) return retval;
    } catch (e) {
        log.error(e);
        return false;
    }

    gItem.name = snippetname.value;
    opener.invalidateItem(gItem);
    gPart.setStringAttribute('name', snippetname.value);
    var indent_relative = indentCheckbox.getAttribute('checked') == 'true';
    if (indent_relative) {
        gPart.setStringAttribute('indent_relative', 'true');
    } else {
        gPart.setStringAttribute('indent_relative', 'false');
    }
    var setSelection = gSetSelectionCheckbox.getAttribute('checked') == 'true';
    if (setSelection) {
        gPart.setStringAttribute('set_selection', 'true');
    } else {
        gPart.setStringAttribute('set_selection', 'false');
        scin.documentEnd();
    }
    // Insert anchor/column markers
    var text = scin.text;
    var anchor = stringutils_charIndexFromPosition(text,scin.anchor);
    var currentPos = stringutils_charIndexFromPosition(text,scin.currentPos);
    if (anchor < currentPos) {
        text = text.slice(0, anchor) + ANCHOR_MARKER +
               text.slice(anchor, currentPos) + CURRENTPOS_MARKER +
               text.slice(currentPos);
    } else {
        text = text.slice(0, currentPos) + CURRENTPOS_MARKER +
               text.slice(currentPos, anchor) + ANCHOR_MARKER +
               text.slice(anchor);
    }
    gPart.value = opener.stringutils_escapeWhitespace(text);

    var iconuri = document.getElementById('snippettab_icon').getAttribute('src');
    gPart.iconurl = iconuri;
    gItem.iconurl = iconuri;

    gApplyButton.setAttribute('disabled', 'true');
    return true;
}


function updateOK() {
    if (snippetname.value == '' || scin.text == '') {
        gOKButton.setAttribute('disabled', 'true');
        gApplyButton.setAttribute('disabled', 'true');
    } else {
        if (gOKButton.hasAttribute('disabled')) {
            gOKButton.removeAttribute('disabled');
        }
        if (gApplyButton.hasAttribute('disabled')) {
            gApplyButton.removeAttribute('disabled');
        }
    }
}

// Do the proper UI updates for a user change.
//  "field" (string) indicates the field to update.
//  "initializing" (boolean, optional) indicates that the dialog is still
//      initializing so some updates, e.g. enabling the <Apply> button, should
//      not be done.
function UpdateField(field, initializing /* =false */)
{
    try {
        if (typeof(initializing) == "undefined" || initializing == null) initializing = false;

        // Only take action if there was an actual change. Otherwise things like
        // the <Alt-A> shortcut when in a textbox will cause a cycle in reenabling
        // the apply button.
        var changed = false;

        switch (field) {
            case 'name':
                var name = snippetname.value;
                if (name) {
                    window.title = "'"+name+"' Properties";
                } else {
                    window.title = "Unnamed " + gPart.prettytype + " Properties";
                }
                snippetnamelabel.value = name;
                changed = true;
                break;
            case 'icon':
                changed = true;
                break;
        }

        if (!initializing && changed) {
            updateOK();
        }
    } catch (e) {
        log.exception(e);
    }
}

function Cancel()  {
    window.arguments[0].res= false;
    return true;
}

function scintillaBlur() {
    scin.focus = true;
    scin.caretPeriod = 0;
}


function scintillaFocus() {
    scin.caretPeriod = 500;
}

function InsertShortcut(shortcutWidget) {
    // Get the shortcut string from the menuitem widget and append it to
    // the current command string.
    scin.replaceSel(shortcutWidget.getAttribute("shortcut"));
    updateOK();
    snippetvalue.setFocus();
}

function update_icon(URI)
{
    try {
        if (URI == 'chrome://jaguar/skin/images/snippet.gif') {
            document.getElementById('reseticon').setAttribute('disabled', 'true');
        } else {
            if (document.getElementById('reseticon').hasAttribute('disabled')) {
                document.getElementById('reseticon').removeAttribute('disabled');
            }
        }
        document.getElementById('keybindingtab_icon').setAttribute('src', URI);
        document.getElementById('snippettab_icon').setAttribute('src', URI);
        if (URI.indexOf('_missing.png') != -1) {
            document.getElementById('snippettab_icon').setAttribute('tooltiptext', "The custom icon specified for this snippet is missing. Please choose another.");
        } else {
            document.getElementById('snippettab_icon').removeAttribute('tooltiptext');
        }
    } catch (e) {
        log.exception(e);
    }
}

function pick_icon(useDefault /* false */)
{
    try {
        var URI
        if (! useDefault) {
            URI = dialog_pickIcon();
            if (!URI) return;
        } else {
            URI = 'chrome://jaguar/skin/images/snippet.gif';
        }
        update_icon(URI);
        updateOK();
    } catch (e) {
        log.exception(e);
    }
}

function doContextMenuHackForPlugin() {
    // for some reason popups over the plugin are messed up until
    // the first context menu over mozilla is activeated.  so we
    // activeate a toolbar context menu, then hide it again, in
    // the hopes that it fixes this problem before the user does
    // something.  we have to fire a mouse event on the toolbar
    // then hide the context menu
    try {
    var toolbar = document.getElementById('spacer');
    var toolbarBox = document.getBoxObjectFor(toolbar);
    var context = document.getElementById('token-context-menu');
    var ev = document.createEvent("MouseEvents");
    ev.initMouseEvent('contextmenu', //in DOMString typeArg,
                true, //in boolean canBubbleArg,
                true, //in boolean cancelableArg,
                gChromeWindowView, //in nsIDOMAbstractView viewArg,
                1, //in long detailArg,
                window.screenX+toolbarBox.x, //in long screenXArg,
                window.screenY+toolbarBox.y, //in long screenYArg,
                toolbarBox.x+1, //in long clientXArg,
                toolbarBox.y+1, //in long clientYArg,
                false, //in boolean ctrlKeyArg,
                false, //in boolean altKeyArg,
                false, //in boolean shiftKeyArg,
                false, //in boolean metaKeyArg,
                2, //in unsigned short buttonArg,
                null //in nsIDOMEventTarget relatedTargetArg
                );
    toolbar.dispatchEvent(ev);
    context.hidePopup();
    } catch(e) {
        log.exception(e);
    }
}
